package cn.iocoder.yudao.module.member.controller.admin.user;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.i18n.core.LangEnum;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditLog.CustomerCreditLogDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customerCreditLog.CustomerCreditLogService;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogCreateReqVO;
import cn.iocoder.yudao.module.ecw.api.internalMessage.ClientInternalMessageApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.dto.InternalMessageCreateDto;
import cn.iocoder.yudao.module.member.api.user.dto.UserRespDTO;
import cn.iocoder.yudao.module.member.api.user.dto.UserSelectReqVO;
import cn.iocoder.yudao.module.member.controller.admin.user.vo.*;
import cn.iocoder.yudao.module.member.convert.user.UserConvert;
import cn.iocoder.yudao.module.member.convert.userCardAuth.UserCardAuthConvert;
import cn.iocoder.yudao.module.member.convert.userEnterpriseAuth.UserEnterpriseAuthConvert;
import cn.iocoder.yudao.module.member.convert.userOperationLog.UserOperationLogConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.dal.dataobject.userCardAuth.UserCardAuthDO;
import cn.iocoder.yudao.module.member.dal.dataobject.userEnterpriseAuth.UserEnterpriseAuthDO;
import cn.iocoder.yudao.module.member.dal.dataobject.userOperationLog.UserOperationLogDO;
import cn.iocoder.yudao.module.member.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.member.enums.UserAuditStatusEnum;
import cn.iocoder.yudao.module.member.service.level.MemberUserLevelConfigService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.service.userCardAuth.UserCardAuthService;
import cn.iocoder.yudao.module.member.service.userEnterpriseAuth.UserEnterpriseAuthService;
import cn.iocoder.yudao.module.member.service.userOperationLog.UserOperationLogService;
import cn.iocoder.yudao.module.member.vo.userCardAuth.UserCardAuthBackVO;
import cn.iocoder.yudao.module.member.vo.userCardAuth.UserCardAuthCreateReqVO;
import cn.iocoder.yudao.module.member.vo.userEnterpriseAuth.UserEnterpriseAuthBackVO;
import cn.iocoder.yudao.module.member.vo.userEnterpriseAuth.UserEnterpriseAuthCreateReqVO;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserLevelConfigBaseRespVO;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserScoreLevelVO;
import cn.iocoder.yudao.module.member.vo.userOperationLog.UserOperationLogBackVO;
import com.alibaba.excel.util.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.*;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

/**
 * @author Administrator
 */
@Api(tags = "用户后台 - 用户操作")
@RestController
@RequestMapping("/member/user")
@Validated
@Slf4j
public class MemberUserController {


    @Resource
    private MemberUserService userService;

    @Resource
    private UserOperationLogService userOperationLogService;

    @Resource
    private UserCardAuthService userCardAuthService;

    @Resource
    private UserEnterpriseAuthService userEnterpriseAuthService;

    @Resource
    private CustomerCreditLogService customerCreditLogService;

    @Resource
    private CustomerService customerService;

    @Resource
    private ClientInternalMessageApi clientInternalMessageApi;


    @Resource
    private PasswordEncoder passwordEncoder;

    @Resource
    private MemberUserLevelConfigService memberUserLevelConfigService;


    @GetMapping("/member-user-list")
    @ApiOperation("获得会员帐号列表-下拉列表使用")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<PageResult<UserRespDTO>> getMemberUserList(@Valid UserSelectReqVO selectVo) {
        PageResult<MemberUserDO> memberUserDOS = userService.selectList(selectVo);
        return success(UserConvert.INSTANCE.converUserRespDTO(memberUserDOS));
    }

    @GetMapping("/get")
    @ApiOperation("获得用户")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('member:user:query')")
    public CommonResult<UserBackVO> getUser(@RequestParam("id") Long id) {
        MemberUserDO user = userService.getUser(id);
        UserBackVO userBackVO = UserConvert.INSTANCE.converUserBackVO(user);
        //判断密码是否是简单密码
        if (passwordEncoder.matches("123456", user.getPassword())) {
            userBackVO.setIsSimplePassword(Boolean.TRUE);
        }
        if (passwordEncoder.matches("88888888", user.getPassword())) {
            userBackVO.setIsSimplePassword(Boolean.TRUE);
        }
        return success(UserConvert.INSTANCE.converUserBackVO(user));
    }


    @GetMapping("/info")
    @ApiOperation("用户详情")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('member:user:info')")
    public CommonResult<UserBackVO> getUserInfo(@RequestParam("id") Long id) {
        MemberUserDO user = userService.info(id);
        UserBackVO userBackVO = UserConvert.INSTANCE.converUserBackVO(user);
        //判断密码是否是简单密码
        if (passwordEncoder.matches("123456", user.getPassword())) {
            userBackVO.setIsSimplePassword(Boolean.TRUE);
        }
        if (passwordEncoder.matches("88888888", user.getPassword())) {
            userBackVO.setIsSimplePassword(Boolean.TRUE);
        }
        MemberUserLevelConfigBaseRespVO  userScoreLevelVO = memberUserLevelConfigService.getLevelByCount(user.getTotalScore());
        MemberUserScoreLevelVO scoreLevelVO = MemberUserScoreLevelVO.builder().memberId(user.getId())
                .memberCode(user.getMemberCode())
                .holdScore(user.getHoldScore())
                .expiredScore(user.getExpiredScore())
                .usedScore(user.getUsedScore())
                .levelIcon(userScoreLevelVO.getIcon())
                .levelName(userScoreLevelVO.getName())
                .build();
        userBackVO.setUserScoreLevelInfo(scoreLevelVO);
        userBackVO.setPassword(null);
        userBackVO.setBackLetterImg(null);
        return success(userBackVO);
    }

    @GetMapping("/list")
    @ApiOperation("获得用户列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('member:user:query')")
    public CommonResult<List<UserBackVO>> getUserList(@RequestParam("ids") Collection<Long> ids) {
        List<MemberUserDO> list = userService.selectList(ids);
        return success(UserConvert.INSTANCE.converUserBackVO(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得用户分页")
    @PreAuthorize("@ss.hasPermission('member:user')")
    public CommonResult<PageResult<UserBackVO>> getUserPage(@Valid UserQueryVO query, PageVO page) {
        PageResult<MemberUserDO> pageResult = userService.getUserPage(query, page);
        return success(UserConvert.INSTANCE.converUserBackVO(pageResult));
    }

    @GetMapping("/waitApprovePage")
    @ApiOperation("获得待审核用户分页")
    //@PreAuthorize("@ss.hasPermission('member:user:query')")
    public CommonResult<PageResult<UserBackVO>> waitApprovePage(@Valid UserQueryVO query, PageVO page) {
        PageResult<MemberUserDO> pageResult = userService.getWaitApproveUserPage(query, page);
        return success(UserConvert.INSTANCE.converUserBackVO(pageResult));
    }

    @GetMapping("/get7daysCount")
    @ApiOperation("获得7天内注册用户统计量")
    //@PreAuthorize("@ss.hasPermission('member:user:query')")
    public CommonResult<List<UserBackVO>> get7daysCount() {
        List<UserBackVO> userBackVOS = userService.get7daysCount();
        return success(userBackVOS);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出用户 Excel")
    @PreAuthorize("@ss.hasPermission('member:export')")
    @OperateLog(type = EXPORT)
    public void exportUserExcel(@Valid UserQueryVO query,
                                HttpServletResponse response) throws IOException {
        List<MemberUserDO> list = userService.getUserList(query);
        // 导出 Excel
        List<UserBackVO> datas = UserConvert.INSTANCE.converUserBackVO(list);
        ExcelUtils.write(response, "用户.xls", "数据", UserBackVO.class, datas);
    }


    @GetMapping("/get-auth-idcard-info")
    @ApiOperation("获取指定用户的实名认证信息")
    @ApiImplicitParam(name = "userId", value = "用户id", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('member:id-card')")
    public CommonResult<UserCardAuthBackVO> getIdcardAuthInfo(@RequestParam("userId") Long userId) {

        UserCardAuthDO one = userCardAuthService.getOne(new LambdaQueryWrapperX<UserCardAuthDO>().eq(UserCardAuthDO::getUserId, userId));

        return success(UserCardAuthConvert.INSTANCE.convert(one));
    }

    @PutMapping("/update-status")
    @ApiOperation("更新用户状态")
    //@PreAuthorize("@ss.hasPermission('member:user:update')")
    public CommonResult<Boolean> updateStatus(@Valid @RequestBody UserUpdateStatusReqVO reqVO) {

        Long id = reqVO.getId();

        MemberUserDO userDO = userService.getUser(id);
        if (userDO == null) {
            return error(ErrorCodeConstants.USER_NOT_EXISTS);
        }

        userDO.setStatus(reqVO.getStatus());
        userService.updateById(userDO);

        return success(true);
    }

    @PutMapping("/create-audit-idcard")
    @ApiOperation("后台管理员 - 新增身份证认证")
    @PreAuthorize("@ss.hasPermission('member:id-card')")
    public CommonResult<Long> createAuditIdcard(@Valid @RequestBody UserCardAuthCreateReqVO reqVO) {
        Long id = userCardAuthService.createUserCardAuth(reqVO, true);
        return success(id);
    }

    @PutMapping("/audit-idcard")
    @ApiOperation("身份证认证审核")
    //@PreAuthorize("@ss.hasPermission('member:user:update')")
    public CommonResult<Boolean> auditIdcard(@Valid @RequestBody UserAuditIdCardReqVO reqVO) {

        Long userCardAuthId = reqVO.getUserCardAuthId();


        UserCardAuthDO userCardAuth = userCardAuthService.getUserCardAuth(userCardAuthId);
        if (userCardAuth == null) {
            return error(ErrorCodeConstants.USER_CARD_AUTH_NOT_EXISTS);
        }

        userCardAuth.setStatus(reqVO.getAuditStatus());
        userCardAuth.setAuditTime(new Date());
        userCardAuth.setAuditRemark(reqVO.getAuditRemark());

        userCardAuthService.updateById(userCardAuth);


        //实名认证审核通过 ，添加信用日志
        if (UserAuditStatusEnum.AUDIT_SUCCESS.getValue().equals(reqVO.getAuditStatus())) {

            //通过memberUserId获取Customer
            CustomerDO customerDO = customerService.getByMemberUserId(userCardAuth.getUserId());

            if (customerDO != null) {
                CustomerCreditLogDO customerCreditLogDO = customerCreditLogService.getOne(new LambdaQueryWrapperX<CustomerCreditLogDO>()
                        .eq(CustomerCreditLogDO::getCustomerId, customerDO.getId())
                        .eq(CustomerCreditLogDO::getRuleId, 1L));
                if (customerCreditLogDO == null) {

                    CustomerCreditLogCreateReqVO customerCreditLogCreateReqVO = new CustomerCreditLogCreateReqVO();
                    customerCreditLogCreateReqVO.setCustomerId(customerDO.getId());
                    //身份证审核
                    customerCreditLogCreateReqVO.setRuleId(1L);

                    //创建身份证审核纪录
                    customerCreditLogService.createCreditLog(customerCreditLogCreateReqVO);
                }

                //如果查询手机相同的客户 审核通过之后自动覆盖手机号相同的客户档案身份证
                if (userCardAuth.getCardType() == 1) {
                    LambdaUpdateWrapper<CustomerDO> lambdaUpdateWrapper = new LambdaUpdateWrapper();
                    lambdaUpdateWrapper.eq(CustomerDO::getId, customerDO.getId());
                    lambdaUpdateWrapper.set(CustomerDO::getCarName, userCardAuth.getName());
                    lambdaUpdateWrapper.set(CustomerDO::getCarNo, userCardAuth.getCardNumber());
                    customerService.update(lambdaUpdateWrapper);
                }


                //站内信
                InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(userCardAuth.getUserId()))
                        .fromId("")
                        .sendTime(new Date())
                        .title(I18nMessage.getMessage("app.message.kyc.success.title", LangEnum.ZH.getLang()))
                        .titleEn(I18nMessage.getMessage("app.message.kyc.success.title", LangEnum.EN.getLang()))
                        .titleFr(I18nMessage.getMessage("app.message.kyc.success.title", LangEnum.FR.getLang()))
                        .content(I18nMessage.getMessage("app.message.kyc.success.content", LangEnum.ZH.getLang()))
                        .contentEn(I18nMessage.getMessage("app.message.kyc.success.content", LangEnum.EN.getLang()))
                        .contentFr(I18nMessage.getMessage("app.message.kyc.success.content", LangEnum.FR.getLang()))
                        .type(3).build();
                clientInternalMessageApi.createInternalMessage(dto);
            }
        } else {
            //审核失败站内信
            InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(userCardAuth.getUserId()))
                    .fromId("")
                    .sendTime(new Date())
                    .title(I18nMessage.getMessage("app.message.kyc.failed.title", LangEnum.ZH.getLang()))
                    .titleEn(I18nMessage.getMessage("app.message.kyc.failed.title", LangEnum.EN.getLang()))
                    .titleFr(I18nMessage.getMessage("app.message.kyc.failed.title", LangEnum.FR.getLang()))
                    .content(I18nMessage.getMessage("app.message.kyc.failed.content", LangEnum.ZH.getLang()) + reqVO.getAuditRemark())
                    .contentEn(I18nMessage.getMessage("app.message.kyc.failed.content", LangEnum.EN.getLang()) + reqVO.getAuditRemark())
                    .contentFr(I18nMessage.getMessage("app.message.kyc.failed.content", LangEnum.FR.getLang()) + reqVO.getAuditRemark())
                    .type(3).build();
            clientInternalMessageApi.createInternalMessage(dto);
        }


        return success(true);
    }


    @PutMapping("/update-idcard")
    @ApiOperation("身份证认证修改")
    //@PreAuthorize("@ss.hasPermission('member:user:update')")
    public CommonResult<Boolean> updateIdcard(@Valid @RequestBody UserCardAuthUpdateReqVO reqVO) {

        Long userCardAuthId = reqVO.getUserCardAuthId();


        UserCardAuthDO userCardAuth = userCardAuthService.getUserCardAuth(userCardAuthId);
        if (userCardAuth == null) {
            return error(ErrorCodeConstants.USER_CARD_AUTH_NOT_EXISTS);
        }

        BeanUtils.copyProperties(reqVO, userCardAuth);

        if (reqVO.isAuditPass()) {
            userCardAuth.setStatus(UserAuditStatusEnum.AUDIT_SUCCESS.getValue());
            userCardAuth.setAuditTime(new Date());
            String auditRemark = userCardAuth.getAuditRemark();
            String auditRemarkNew = (StringUtils.isNotEmpty(auditRemark) ? (auditRemark + "; ") : "") + "修改后审核通过";
            userCardAuth.setAuditRemark(auditRemarkNew);

            //站内信
            InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(userCardAuthId))
                    .fromId("")
                    .sendTime(new Date())
                    .title(I18nMessage.getMessage("app.message.kyc.success.title", LangEnum.ZH.getLang()))
                    .titleEn(I18nMessage.getMessage("app.message.kyc.success.title", LangEnum.EN.getLang()))
                    .titleFr(I18nMessage.getMessage("app.message.kyc.success.title", LangEnum.FR.getLang()))
                    .content(I18nMessage.getMessage("app.message.kyc.success.content", LangEnum.ZH.getLang()))
                    .contentEn(I18nMessage.getMessage("app.message.kyc.success.content", LangEnum.EN.getLang()))
                    .contentFr(I18nMessage.getMessage("app.message.kyc.success.content", LangEnum.FR.getLang()))
                    .type(3).build();
            clientInternalMessageApi.createInternalMessage(dto);
        }
        userCardAuthService.updateById(userCardAuth);

        return success(true);
    }


    @GetMapping("/get-auth-enterprise-info")
    @ApiOperation("获取指定用户提交的企业认证信息")
    @ApiImplicitParam(name = "userId", value = "用户id", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('member:business-license')")
    public CommonResult<UserEnterpriseAuthBackVO> getEnterpriseAuthInfo(@RequestParam("userId") Long userId) {

        UserEnterpriseAuthDO one = userEnterpriseAuthService.getOne(new LambdaQueryWrapperX<UserEnterpriseAuthDO>().eq(UserEnterpriseAuthDO::getUserId, userId));

        return success(UserEnterpriseAuthConvert.INSTANCE.convert(one));
    }

    @PutMapping("/create-audit-enterprise")
    @ApiOperation("后台管理员 - 新增企业认证")
    @PreAuthorize("@ss.hasPermission('member:business-license')")
    public CommonResult<Long> createAuditEnterprise(@Valid @RequestBody UserEnterpriseAuthCreateReqVO reqVO) {
        Long id = userEnterpriseAuthService.createUserEnterpriseAuth(reqVO, true);
        return success(id);
    }

    @PutMapping("/audit-enterprise")
    @ApiOperation("企业认证审核")
    //@PreAuthorize("@ss.hasPermission('member:user:update')")
    public CommonResult<Boolean> auditEnterprise(@Valid @RequestBody UserAuditEnterpriseReqVO reqVO) {

        Long enterpriseAuthId = reqVO.getUserEnterpriseAuthId();


        UserEnterpriseAuthDO userEnterpriseAuth = userEnterpriseAuthService.getUserEnterpriseAuth(enterpriseAuthId);
        if (userEnterpriseAuth == null) {
            return error(ErrorCodeConstants.USER_ENTERPRISE_AUTH_NOT_EXISTS);
        }

        userEnterpriseAuth.setStatus(reqVO.getAuditStatus());
        userEnterpriseAuth.setAuditTime(new Date());
        userEnterpriseAuth.setAuditRemark(reqVO.getAuditRemark());

        userEnterpriseAuthService.updateById(userEnterpriseAuth);

        return success(true);
    }

    @PutMapping("/update-enterprise")
    @ApiOperation("企业认证修改")
    //@PreAuthorize("@ss.hasPermission('member:user:update')")
    public CommonResult<Boolean> updateEnterprise(@Valid @RequestBody UserEnterpriseAuthUpdateReqVO reqVO) {

        Long enterpriseAuthId = reqVO.getUserEnterpriseAuthId();


        UserEnterpriseAuthDO userEnterpriseAuth = userEnterpriseAuthService.getUserEnterpriseAuth(enterpriseAuthId);
        if (userEnterpriseAuth == null) {
            return error(ErrorCodeConstants.USER_ENTERPRISE_AUTH_NOT_EXISTS);
        }

        BeanUtils.copyProperties(reqVO, userEnterpriseAuth);

        if (reqVO.isAuditPass()) {
            userEnterpriseAuth.setStatus(UserAuditStatusEnum.AUDIT_SUCCESS.getValue());
            userEnterpriseAuth.setAuditTime(new Date());
            String auditRemark = userEnterpriseAuth.getAuditRemark();
            String auditRemarkNew = (StringUtils.isNotEmpty(auditRemark) ? (auditRemark + "; ") : "") + "修改后审核通过";
            userEnterpriseAuth.setAuditRemark(auditRemarkNew);
        }

        userEnterpriseAuthService.updateById(userEnterpriseAuth);

        return success(true);
    }

    @PutMapping("/update-backletter")
    @ApiOperation("更新保函")
    @PreAuthorize("@ss.hasPermission('member:certificate')")
    public CommonResult<Boolean> updateBackletter(@Valid @RequestBody UseBackLetterReqVO reqVO) {


        Long userId = reqVO.getUserId();


        MemberUserDO user = userService.getUser(userId);
        if (user == null) {
            return error(ErrorCodeConstants.USER_NOT_EXISTS);
        }

        user.setBackLetterImg(reqVO.getBackLetter());

        userService.updateById(user);

        return success(true);
    }


    @GetMapping("/operation-log/page")
    @ApiOperation("获得用户操作日志分页")
    @PreAuthorize("@ss.hasPermission('member:log')")
    public CommonResult<PageResult<UserOperationLogBackVO>> selectUserOperationLogPage(@Valid UserOperationLogQueryVO query, PageVO page) {
        PageResult<UserOperationLogDO> pageResult = userOperationLogService.selectPage(page, new LambdaQueryWrapperX<UserOperationLogDO>()
                .eqIfPresent(UserOperationLogDO::getUserId, query.getUserId()));
        return success(UserOperationLogConvert.INSTANCE.convertPage(pageResult));
    }


    @DeleteMapping("/delete")
    @ApiOperation("删除注册会员")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('member:delete')")
    public CommonResult<Boolean> deleteUser(@RequestParam("id") Long id) {
        userService.deleteById(id);
        return success(true);
    }


    @GetMapping("/backLetterImg")
    @ApiOperation("获得用户保函/证书")
    @ApiImplicitParam(name = "id", value = "用户编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('member:backLetterImg')")
    public CommonResult<String> getBackLetterImg(@RequestParam("id") Long id) {
        MemberUserDO user = userService.getUser(id);
        if (Objects.nonNull(user)){
            return success(user.getBackLetterImg());
        }
        return success(null);
    }

    @PutMapping("/memberCode/flush")
    @ApiOperation("刷新历史会员编码")
    public CommonResult<String> memberCodeFlush() {
        userService.historyCodeFlush();
        return success(null);
    }
}
